-- VECTRIC LUA SCRIPT
-- Copyright 2014 Vectric Ltd.

-- Gadgets are an entirely optional add-in to Vectric's core software products. 
-- They are provided 'as-is', without any express or implied warranty, and you make use of them entirely at your own risk.
-- In no event will the author(s) or Vectric Ltd. be held liable for any damages arising from their use.

-- Permission is granted to anyone to use this software for any purpose, 
-- including commercial applications, and to alter it and redistribute it freely, 
-- subject to the following restrictions:

-- 1. The origin of this software must not be misrepresented; you must not claim that you wrote the original software. If you use this software in a product, an acknowledgment in the product documentation would be appreciated but is not required.
-- 2. Altered source versions must be plainly marked as such, and must not be misrepresented as being the original software.
-- 3. This notice may not be removed or altered from any source distribution.

require "strict"
--require("mobdebug").start()

g_title = "Replace Below"
g_version = "1.2"

--[[  ------------------------ ReplaceReliefBelow --------------------------------------------------
|
| Makes the passed relief within the passed height range transparent
|
|  Parameters
|  relief              -- the relief to modify
|  min_z               -- the z value below which we modify the data
|  new_z	   		   -- the new z value we assign to the filtered points
|  shift_z			   -- if the z value at a point survives the filter
|						  then shift it by this amount
|
]]
function ReplaceReliefBelow(relief, min_z, new_z, shift_z)
  
  -- Check the inputs
  if not relief then
	return
  end
  
  -- Get the dimensions of the relief
  local pixel_width = relief.PixelWidth
  local pixel_height = relief.PixelHeight
	
  -- Set up a progress bar
  local progress_bar = ProgressBar("Processing", ProgressBar.LINEAR)
	
  -- For each non transparent pixel in the relief subract the z value from max_z
  -- inverting the relief's appearance
  for y = 0, pixel_height - 1 do
    for x = 0, pixel_width - 1 do
      local z = relief:Get(x, y)
     		if z <= min_z then
			z = new_z
		end
		if not IsTransparent(z) then
			z = z + shift_z
		end
		relief:Set(x, y, z)
    end
    -- Step progress
    if pixel_height > 1 then
      local progress = y / (pixel_height - 1);
      progress_bar:SetPercentProgress(100 * progress + 0.5)
    end	
  end
  
  -- Tell the progress bar we're done
  progress_bar:Finished() 
  
end

--[[  ------------------------ OnLuaButton_ShiftZCheck --------------------------------------------------
|
|	Dummy callback
]]
function OnLuaButton_ShiftZCheck()
return true
end

--[[  -------------- DisplayDialog --------------------------------------------------
|
|  Displays the options dialog
|  Parameters:
|	options           -- The set of options for dialog
|   script_path       -- Where to locate the html pages
|   job            	  -- The job
|     
|
|  Return Values:
|  1 on success,
|  0 if the user cancelled
|  -1 on bad input
|
]]
function DisplayDialog(options, script_path, job)

  -- Prepare the dialog
  local script_html = "file:" .. script_path .. "\\Replace_Below.htm"
  local dialog_name = options.gadget_title
  local dialog = HTML_Dialog(false, script_html, options.window_width, options.window_height, dialog_name)
  
  -- Set the gadget name and version
  dialog:AddLabelField("GadgetTitle", options.gadget_title)
  dialog:AddLabelField("GadgetVersion", options.gadget_version)
  
  -- Add the min_z field
  dialog:AddDoubleField("MinZEdit", options.min_z)
  -- Add the type of replacement
  dialog:AddRadioGroup("NewZValue", 1)
  -- Do we lower the relief too
  dialog:AddCheckBox("ShiftZCheck", false)
  
  -- Show the dialog
  if not dialog:ShowDialog() then
    return 0
  end
	
  -- Update the options from this dialog
  options.min_z = dialog:GetDoubleField("MinZEdit")
  
  local new_z_index = dialog:GetRadioIndex("NewZValue")
  options.new_z = (new_z_index == 1) and GetTransparentHeight() or options.min_z
  
  local shift_z  = dialog:GetCheckBox("ShiftZCheck")
  if shift_z then
	options.shift_z = -options.min_z
  end
  
  return 1
end


--[[  ------------------------ main --------------------------------------------------
|
|  Entry point for script
|
]]
function main(script_path)
  
  -- Check for Aspire
  if not IsAspire() then
	MessageBox("This gadget requires Aspire!")
	return false
  end
  
  if GetBuildVersion() < 7.5 then
	MessageBox("This gadget requires Aspire 4.5 or later!")
	return false
  end
  
  -- Check for the existence of a job
  local job = VectricJob()
  if not job.Exists then
	MessageBox("No job created, please start a new job and re-run the gadget")
    return false
  end
  
  -- Get the component manager 
  local component_manager = job.ComponentManager
  if not component_manager then
	MessageBox("No Component Manager!")
	return false
  end

  -- Get the list of selected components (top level only)
  local selected_ids = component_manager:GetSelectedObjectIds(false)
  if not selected_ids or selected_ids.IsEmpty then
	MessageBox("No components selected, please select your component and re-run the gadget")
	return false
  end
  
  local transparent = GetTransparentHeight()
   -- Set up our options and display the dialog
  local options = {
    gadget_title = g_title,
    gadget_version = g_version,
    window_width = 640,
    window_height = 320,
	min_z = 0.0,       
 	new_z = 0.0, 
	shift_z = 0.0
	}
  
  -- Loop until either the user cancels or presses ok
  local dialog_code = -1
  while dialog_code == -1 do
    dialog_code = DisplayDialog(options, script_path, job)
  end
  
  -- Did the user cancel
  if dialog_code == 0 then
    return false
  end
  
  -- Loop over all the components in the selection
  local pos = selected_ids:GetHeadPosition()
  while pos do
    local id = nil
    id, pos = selected_ids:GetNext(pos)
      local baked_id = component_manager:CreateBakedCopyOfObject(id, true) -- 
      local component = component_manager:FindObjectWithId(baked_id)
    -- Should have a component by now
    if component then
      local relief = component:GetRelief()
      ReplaceReliefBelow(relief, options.min_z, options.new_z, options.shift_z)
    else
      MessageBox("No component(s) selected!")
    end
  end
  
  -- Update all the previews now
  component_manager:UpdatePreviews()
  
  -- Update the composite model
  component_manager:UpdateCompositeModel()
  
  return true
  
end
